/*
 *
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Module: libdrivelink.so
 *
 *   File: dm.c
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <plugin.h>

#include "drivelink.h"
#include "dm.h"


/*
 *  Function: DL_activate
 *
 *  Called in order to create a mapping of an evms drivelink
 *  in the kernel.
 */
int DL_activate( storage_object_t *drivelink )
{
        int rc=EINVAL, i;
        Drive_Link_Private_Data *pdata=NULL;
        dm_target_t *target, *target_list = NULL;
        dm_device_t *dev;

        LOGENTRY();

        pdata = (Drive_Link_Private_Data *)drivelink->private_data;

        if ((pdata) && (pdata->signature == EVMS_DRIVELINK_SIGNATURE)) {

                if (pdata->drive_link_count) {

                        for (i=0,rc=0; (!rc) && (i<pdata->drive_link_count); i++) {
                                target = DLEngFncs->dm_allocate_target(DM_TARGET_LINEAR,
                                                                       pdata->drive_link[i].start_lsn,
                                                                       pdata->drive_link[i].sector_count, 0);
                                if (target) {
                                        dev = target->data.linear;
                                        dev->major = pdata->drive_link[i].object->dev_major;
                                        dev->minor = pdata->drive_link[i].object->dev_minor;
                                        dev->start = 0;
                                        DLEngFncs->dm_add_target(target, &target_list);
                                } else {
                                        rc = ENOMEM;
                                }
                        }

                        if (!rc) {

                                rc = DLEngFncs->dm_activate(drivelink, target_list);

				if (!rc) {
					drivelink->flags |=  SOFLAG_ACTIVE;
					drivelink->flags &= ~SOFLAG_NEEDS_ACTIVATE;
				}

                        }

                        DLEngFncs->dm_deallocate_targets(target_list);
                }
        }

        LOGEXITRC();
        return rc;
}


/*
 *  Function: DL_deactivate
 *
 *  Called in order to delete an existing mapping of an evms
 *  drivelink storage object in the kernel.
 */
int DL_deactivate( storage_object_t *drivelink )
{
        int rc;

        LOGENTRY();

        rc = DLEngFncs->dm_deactivate(drivelink);

	if (!rc) {
		drivelink->flags &= ~SOFLAG_ACTIVE;
	}

        LOGEXITRC();
        return rc;
}

